#include "apicompattests.h"
#include "../api/meteorddp.h"
#include "../ddpRequests/ddploginrequest.h"
#include "../ddpRequests/rocketchatblockuserrequest.h"
#include "../ddpRequests/rocketchatunblockuserrequest.h"
#include "../ddpRequests/rocketchatchangeuserpresancedefaultstatus.h"
#include "../ddpRequests/ddpsubscriptionrequest.h"
#include "../ddpRequests/rocketchatsubscribeactiveusers.h"
#include "../ddpRequests/rocketchatsubscribechannelrequest.h"
#include "../ddpRequests/rocketchatsubscribeusernotify.h"
#include "../ddpRequests/rocketchatsubscriberoomrequest.h"
#include "../ddpRequests/rocketchatnotifynoticesrequest.h"
#include "../ddpRequests/rocketchatsubscriptionchangedrequest.h"
#include "../ddpRequests/rocketchatcreatechannelrequest.h"
#include "../ddpRequests/rocketchatcreateprivategrouprequest.h"
#include "../rocketchatserverconfig.h"

#include <QtTest/QtTest>
#include <QSignalSpy>
#include <QJsonObject>

void ApiCompatTests::initTestCase()
{
    std::get<0>( RocketChatServerConfig::serverVersion ) = MAJOR;
    std::get<1>( RocketChatServerConfig::serverVersion ) = MINOR;
    std::get<2>( RocketChatServerConfig::serverVersion ) = PATCH;
    mDDPApi = new MeteorDDP( this, "rocketchat:3000", false );
    QSignalSpy spy( mDDPApi, SIGNAL( websocketConnected() ) );
    QVERIFY( spy.wait() );
}

void ApiCompatTests::cleanupTestCase()
{
    mDDPApi->disconnectFromServer();
    mDDPApi->deleteLater();
}

void ApiCompatTests::testConnectToServer()
{
    /*mDDPApi->connectWithServer();

    QSignalSpy spy( mDDPApi, SIGNAL( ddpConnected() ) );

    QVERIFY( spy.wait( ) );*/
}

void ApiCompatTests::testLoginWithSHA256()
{
    QByteArray hashArray = QCryptographicHash::hash( "123", QCryptographicHash::Sha256 );
    QString pass = hashArray.toHex();
    QString username = "testUser0";

    bool loginSuccess = false;

    std::function<void ( QJsonObject, MeteorDDP * )> success = [ & ]( QJsonObject pResponse, MeteorDDP * ) {
        qDebug() << "authenticated";

        if ( pResponse.contains( QStringLiteral( "result" ) ) ) {
            QJsonObject result = pResponse[QStringLiteral( "result" )].toObject();

            if ( result.contains( QStringLiteral( "token" ) ) && result.contains( QStringLiteral( "id" ) ) && result.contains( QStringLiteral( "tokenExpires" ) ) )  {
                QJsonObject expireObject = result[( "tokenExpires" )].toObject();

                if ( expireObject.contains( QStringLiteral( "$date" ) ) ) {
                    if ( expireObject[QStringLiteral( "$date" )].isDouble() ) {
                        loginSuccess = true;
                        this->mSessionToken = result["token"].toString();
                        this->mUserId = result[QStringLiteral( "id" )].toString();
                    }
                }
            }
        }
    };

    auto loginRequest = QSharedPointer<DDPLoginRequest>( new DDPLoginRequest( username, pass ) );
    loginRequest->setSuccess( success );
    mDDPApi->sendRequest( loginRequest );

    QTest::qWait( 200 );

    QVERIFY( loginSuccess );
}

void ApiCompatTests::testLoginWithSessionToken()
{
    bool loginSuccess = false;

    std::function<void ( QJsonObject, MeteorDDP * )> success = [ & ]( QJsonObject pResponse, MeteorDDP * ) {
        qDebug() << "authenticated";

        if ( pResponse.contains( QStringLiteral( "result" ) ) ) {
            QJsonObject result = pResponse[QStringLiteral( "result" )].toObject();

            if ( result.contains( QStringLiteral( "token" ) ) && result.contains( QStringLiteral( "id" ) ) && result.contains( QStringLiteral( "tokenExpires" ) ) )  {
                QJsonObject expireObject = result[( "tokenExpires" )].toObject();

                if ( expireObject.contains( QStringLiteral( "$date" ) ) ) {
                    if ( expireObject[QStringLiteral( "$date" )].isDouble() ) {
                        loginSuccess = true;
                        this->mSessionToken = result["token"].toString();
                        this->mUserId = result[QStringLiteral( "id" )].toString();
                    }
                }
            }
        }
    };

    auto loginRequest = QSharedPointer<DDPLoginRequest>( new DDPLoginRequest( mSessionToken ) );
    loginRequest->setSuccess( success );
    mDDPApi->sendRequest( loginRequest );

    QTest::qWait( 200 );

    QVERIFY( loginSuccess );
}

void ApiCompatTests::testDDPSubscriptions()
{
    QSharedPointer<RocketChatSubscribeUserNotify> notifySubSubscription( new RocketChatSubscribeUserNotify( this->mUserId ) );
    QSharedPointer<RocketChatNotifyNoticesRequest> notifyNoticeSubSubscription( new RocketChatNotifyNoticesRequest( this->mUserId ) );
    QSharedPointer<RocketChatSubScriptionChangedRequest> noitfySubscriptionsSubscription( new RocketChatSubScriptionChangedRequest( this->mUserId ) );
    QSharedPointer<RocketChatSubscribeActiveUsers> activeUsersSubscription( new RocketChatSubscribeActiveUsers() );
    QSharedPointer<RocketChatSubscribeChannelRequest> testUser1ChannelSub( new RocketChatSubscribeChannelRequest( "AX8HDAKASKki6ZjDJwakuwFBdwjcSuan2n" ) );


    mDDPApi->sendRequest( notifySubSubscription );
    mDDPApi->sendRequest( notifyNoticeSubSubscription );
    mDDPApi->sendRequest( noitfySubscriptionsSubscription );
    mDDPApi->sendRequest( activeUsersSubscription );
    mDDPApi->sendRequest( testUser1ChannelSub );
}

void ApiCompatTests::testFileUpload()
{

}

void ApiCompatTests::testAddUserToChannel()
{

}

void ApiCompatTests::testBlockUser()
{
    bool success = false;
    QSharedPointer<RocketChatBlockUserRequest> blockRequest( new RocketChatBlockUserRequest( "AX8HDAKASKki6ZjDJwakuwFBdwjcSuan2n", "wakuwFBdwjcSuan2n" ) );
    connect( mDDPApi, &MeteorDDP::messageReceived, [&]( QJsonObject pMessage ) {
        if ( pMessage.contains( "collection" ) && pMessage["collection"] == "stream-notify-user" && pMessage.contains( "fields" ) ) {
            QJsonObject fields = pMessage["fields"].toObject();

            if ( fields.contains( "args" ) ) {
                QJsonArray args = fields[ "args" ].toArray();
                QString firstArg = args[0].toString();

                if ( firstArg == "updated" )   {
                    QJsonObject data = args[1].toObject();

                    if ( data["rid"] == "AX8HDAKASKki6ZjDJwakuwFBdwjcSuan2n" ) {
                        if ( data.contains( "blocker" ) ) {
                            if ( data["blocker"].toBool() ) {
                                success = true;
                            }
                        }
                    }
                }
            }
        }
    } );
    mDDPApi->sendRequest( blockRequest );
    QTest::qWait( 200 );
    QVERIFY( success );
}

void ApiCompatTests::testUnBlockUser()
{
    bool success = false;
    QSharedPointer<RocketChatUnblockUserRequest> blockRequest( new RocketChatUnblockUserRequest( "AX8HDAKASKki6ZjDJwakuwFBdwjcSuan2n", "wakuwFBdwjcSuan2n" ) );
    connect( mDDPApi, &MeteorDDP::messageReceived, [&]( QJsonObject pMessage ) {
        if ( pMessage.contains( "collection" ) && pMessage["collection"] == "stream-notify-user" && pMessage.contains( "fields" ) ) {
            QJsonObject fields = pMessage["fields"].toObject();

            if ( fields.contains( "args" ) ) {
                QJsonArray args = fields["args" ].toArray();

                QString firstArg = args[0].toString();

                if ( firstArg == "updated" )   {
                    QJsonObject data = args[1].toObject();

                    if ( data["rid"] == "AX8HDAKASKki6ZjDJwakuwFBdwjcSuan2n" ) {
                        if ( data.contains( "blocker" ) ) {
                            if ( !data["blocker"].toBool() ) {
                                success = true;
                            }
                        } else {
                            success = true;
                        }
                    }
                }
            }
        }
    } );
    mDDPApi->sendRequest( blockRequest );
    QTest::qWait( 200 );
    QVERIFY( success );
}

void ApiCompatTests::testChangeUserPresenceDefault()
{
    bool success = false;
    connect( mDDPApi, &MeteorDDP::messageReceived, [&]( QJsonObject pMessage ) {
        if ( pMessage.contains( "collection" ) && pMessage["collection"] == "users" && pMessage.contains( "fields" ) ) {
            QJsonObject fields = pMessage["fields"].toObject();

            if ( pMessage["id"].toString() == "AX8HDAKASKki6ZjDJ" && fields.contains( "statusDefault" ) ) {
                QString statusDefault = fields["statusDefault" ].toString();

                if ( statusDefault == "away" ) {
                    success = true;
                }

            }
        }
    } );
    QSharedPointer<DDPMethodRequest> request( new RocketChatChangeUserPresanceDefaultStatus( static_cast<RocketChatChangeUserPresanceDefaultStatus::status>( 2 ) ) );
    mDDPApi->sendRequest( request );

    QTest::qWait( 200 );

    QVERIFY( success );
}

void ApiCompatTests::testCreateChannel()
{
    bool check = false;
    DdpCallback success = [ & ]( QJsonObject pResponse, MeteorDDP * pDdpApi ) {
        Q_UNUSED( pDdpApi );

        if ( pResponse.contains( "rid" ) && pResponse.contains( "name" ) ) {
            check = true;
        }
    };
    QStringList userList;
    userList.append( "testUser2" );
    QSharedPointer<RocketChatCreateChannelRequest> request( new RocketChatCreateChannelRequest( "testChannel0", userList, false ) );
    request->setSuccess( success );
    mDDPApi->sendRequest( request );

    QTest::qWait( 200 );
    QVERIFY( check );
}

void ApiCompatTests::testCreatePrivateGroup()
{
    bool check = false;
    DdpCallback success = [ & ]( QJsonObject response, MeteorDDP * ddpApi ) {
        Q_UNUSED( ddpApi );

        if ( response.contains( "rid" ) && response.contains( "name" ) ) {
            check = true;
        }
    };
    QStringList userList;
    userList.append( "testUser2" );
    QSharedPointer<RocketChatCreatePrivateGroupRequest> request( new RocketChatCreatePrivateGroupRequest( "testPrivChannel0", userList, false ) );
    request->setSuccess( success );
    mDDPApi->sendRequest( request );

    QTest::qWait( 200 );
    QVERIFY( check );
}

void ApiCompatTests::testSendTextMessage()
{
    bool success = false;
    connect( mDDPApi, &MeteorDDP::messageReceived, [&]( QJsonObject pMsg ) {
        if ( pMsg.contains( "msg" ) && pMsg["msg"] == "changed" && pMsg.contains( "collection" ) && pMsg["collection"] == "stream-room-messages" && pMsg.contains( "id" ) && pMsg.contains( "fields" ) ) {
            QJsonObject fields = pMsg["fields"].toObject();

            if ( fields.contains( "args" ) ) {
                QJsonArray args = fields["args"].toArray();
                QJsonObject firstArg = args[0].toObject();

                if ( firstArg.contains( "rid" ) && firstArg.contains( "msg" ) && firstArg.contains( "ts" ) && firstArg.contains( "u" ) ) {
                    QString rid = firstArg["rid"].toString();
                    QString msg = firstArg["msg"].toString();
                    QJsonObject userObj = firstArg["u"].toObject();
                    QString userId = userObj["_id"].toString();

                    if ( rid == "AX8HDAKASKki6ZjDJwakuwFBdwjcSuan2n" && msg == "testMessage1" && userId == mUserId ) {
                        success = true;
                    }
                }
            }
        }
    } );
    QJsonObject params;
    params[QStringLiteral( "rid" )] = "AX8HDAKASKki6ZjDJwakuwFBdwjcSuan2n";
    params[QStringLiteral( "msg" )] = "testMessage1";
    QSharedPointer<DDPRequest> sendMessageRequest( new DDPMethodRequest( QStringLiteral( "sendMessage" ), {params} ) );
    mDDPApi->sendRequest( sendMessageRequest );

    QTest::qWait( 200 );
    QVERIFY( success );
}

void ApiCompatTests::testGetRooms()
{

}

void ApiCompatTests::testHideRooms()
{

}

void ApiCompatTests::testLeaveRooms()
{

}

void ApiCompatTests::tetMarkChannelAsRead()
{

}

void ApiCompatTests::testSearchMessage()
{

}

void ApiCompatTests::testSubscribeUserNotice()
{

}

void ApiCompatTests::testSearchUsers()
{

}

void ApiCompatTests::testSubscribeActiveUsers()
{

}

void ApiCompatTests::testSubscribeChannel()
{

}

void ApiCompatTests::testSubscribeLoginMethods()
{

}

void ApiCompatTests::testSubscribeUserNotify()
{

}

void ApiCompatTests::testSubscriptionChanged()
{

}

void ApiCompatTests::testUpdateJitsiMeetTimeout()
{

}

void ApiCompatTests::testUpdatePushToken()
{

}
