/********************************************************************************************
 *                                                                                          *
 * Copyright (C) 2017 Armin Felder, Dennis Beier                                            *
 * This file is part of RocketChatMobileEngine <https://git.fairkom.net/chat/fairchat>.     *
 *                                                                                          *
 * RocketChatMobileEngine is free software: you can redistribute it and/or modify           *
 * it under the terms of the GNU General Public License as published by                     *
 * the Free Software Foundation, either version 3 of the License, or                        *
 * (at your option) any later version.                                                      *
 *                                                                                          *
 * RocketChatMobileEngine is distributed in the hope that it will be useful,                *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                            *
 * GNU General Public License for more details.                                             *
 *                                                                                          *
 * You should have received a copy of the GNU General Public License                        *
 * along with RocketChatMobileEngine. If not, see <http://www.gnu.org/licenses/>.           *
 *                                                                                          *
 ********************************************************************************************/



#include <QUrl>
#include <QThread>
#include <QDebug>

#include "websocket.h"
#ifdef Q_OS_WINRT
#include "websocket/winrtwebsocket/websocketwinrt.h"
#else
#include "websocketlinux.h"
#endif

Websocket::Websocket( QObject *parent ): WebsocketAbstract( parent )
{
    qRegisterMetaType<QAbstractSocket::SocketState>( "QAbstractSocket::SocketState" );
#ifdef Q_OS_WINRT
    qDebug() << "Windows RT websocket instantiated";
    mWebsocketInstance = new WinRTWebsocket;
#else
    mWebsocketInstance = new websocketLinux( this );
#endif
    connect( mWebsocketInstance, &WebsocketAbstract::connected, this, &Websocket::connected, Qt::UniqueConnection );
    connect( mWebsocketInstance, &WebsocketAbstract::textMessageReceived, this, &Websocket::textMessageReceived, Qt::UniqueConnection );
    connect( mWebsocketInstance, &WebsocketAbstract::error, this, &Websocket::error, Qt::UniqueConnection );
    connect( mWebsocketInstance, &WebsocketAbstract::closed, this, &Websocket::closed, Qt::UniqueConnection );
}

void Websocket::open( QUrl pUrl )
{
    mWebsocketInstance->open( pUrl );
}

void Websocket::close()
{
    mWebsocketInstance->close();
}

void Websocket::sendTextMessage( QString pMessage )
{
    mWebsocketInstance->sendTextMessage( pMessage );
}

bool Websocket::isValid()
{
    return mWebsocketInstance->isValid();
}

bool Websocket::isDisconnected()
{
    return mWebsocketInstance->isDisconnected();
}

bool Websocket::isConnecting()
{
    return mWebsocketInstance->isConnecting();
}


void Websocket::updateConnectionStatus()
{
    qDebug() << "updated connection status";
    mConnectionStatus = true;
    emit( connected() );
}
/*
Websocket::~Websocket(){
    qDebug()<<"destroying websocket instance";
    disconnect(mWebsocketInstance,&WebsocketAbstract::connected,this,&Websocket::connected);
    disconnect(mWebsocketInstance,&WebsocketAbstract::textMessageReceived,this,&Websocket::textMessageReceived);
    disconnect(mWebsocketInstance,&WebsocketAbstract::error,this,&Websocket::error);
    disconnect(mWebsocketInstance,&WebsocketAbstract::closed,this,&Websocket::closed);
    mWebsocketInstance->close();
    mWebsocketInstance->deleteLater();

}*/
